@@ -46,19 +46,29 @@
 #define __STDC_CONSTANT_MACROS
 
 // LLVM includes
+#if HS_LLVM_VERSION < 303
 #include "llvm/LLVMContext.h"
-#include "llvm/Bitcode/ReaderWriter.h"
-#include "llvm/Support/MemoryBuffer.h"
-#include "llvm/Support/Casting.h"
 #include "llvm/Constants.h"
 #include "llvm/DerivedTypes.h"
 #include "llvm/GlobalVariable.h"
+#include "llvm/IntrinsicInst.h"
+#else
+#include "llvm/IR/LLVMContext.h"
+#include "llvm/IR/Module.h"
+#include "llvm/IR/IRBuilder.h"
+#include "llvm/IR/Constants.h"
+#include "llvm/IR/DerivedTypes.h"
+#include "llvm/IR/GlobalVariable.h"
+#include "llvm/IR/IntrinsicInst.h"
+#endif
+#include "llvm/Bitcode/ReaderWriter.h"
+#include "llvm/Support/MemoryBuffer.h"
+#include "llvm/Support/Casting.h"
 #if HS_LLVM_VERSION < 300
 #include "llvm/TypeSymbolTable.h"
 #endif
 #include "llvm/Support/MemoryBuffer.h"
 #include "llvm/Support/CallSite.h"
-#include "llvm/IntrinsicInst.h"
 #include "llvm/Analysis/Verifier.h"
 #include "llvm/Assembly/Parser.h"
 #ifdef HAVE_LLVM_SUPPORT_DYNAMICLIBRARY_H
@@ -79,6 +89,27 @@
 #include "llvm/Linker.h"
 #include "llvm/Support/SourceMgr.h"
 
+#if HS_LLVM_VERSION >= 300
+// Imports for direct object emission
+// Target selection
+#if HS_LLVM_VERSION < 302
+#include "llvm/Target/TargetData.h"
+#elif HS_LLVM_VERSION < 303
+#include "llvm/DataLayout.h"
+#else
+#include "llvm/IR/DataLayout.h"
+#endif
+#include "llvm/Target/TargetMachine.h"
+#include "llvm/Support/TargetSelect.h"
+#include "llvm/Support/TargetRegistry.h"
+#include "llvm/Support/Host.h"
+#include "llvm/ADT/SmallVector.h"
+
+// File output
+#include "llvm/Support/raw_ostream.h"
+#include "llvm/Support/FormattedStream.h"
+#endif
+
 // LLVM-C includes
 #include "llvm-c/Core.h"
 #include "llvm-c/ExecutionEngine.h"
@@ -192,12 +223,14 @@ const char *LLVMInstGetOpcodeName(LLVMValueRef inst)
     return instp->getOpcodeName();
 }
 
+#if HS_LLVM_VERSION < 301
 unsigned LLVMInstGetOpcode(LLVMValueRef inst)
 {
     llvm::Instruction *instp = llvm::unwrap<llvm::Instruction>(inst);
     assert(instp);
     return instp->getOpcode();
 }
+#endif
 
 unsigned LLVMCmpInstGetPredicate(LLVMValueRef cmpinst)
 {
@@ -341,12 +374,12 @@ unsigned LLVMGetDoesNotThrow(LLVMValueRef fn)
     return fnp->doesNotThrow();
 }
 
-void LLVMSetDoesNotThrow(LLVMValueRef fn, int DoesNotThrow)
+void LLVMSetDoesNotThrow(LLVMValueRef fn)
 {
     llvm::Function *fnp = llvm::unwrap<llvm::Function>(fn);
     assert(fnp);
 
-    return fnp->setDoesNotThrow((bool)DoesNotThrow);
+    return fnp->setDoesNotThrow();
 }
 
 LLVMValueRef LLVMGetIntrinsic(LLVMModuleRef module, int id,
@@ -387,7 +420,7 @@ LLVMModuleRef LLVMGetModuleFromAssembly(const char *asmtext, unsigned txtlen,
                                               llvm::getGlobalContext()))) {
         std::string s;
         llvm::raw_string_ostream buf(s);
-        error.Print("llvm-py", buf);
+        error.print("llvm-py", buf);
         *out = strdup(buf.str().c_str());
         return NULL;
     }
@@ -417,6 +450,7 @@ LLVMModuleRef LLVMGetModuleFromBitcode(const char *bitcode, unsigned bclen,
     return wrap(modulep);
 }
 
+#if HS_LLVM_VERSION < 302
 unsigned LLVMLinkModules(LLVMModuleRef dest, LLVMModuleRef src, unsigned mode,
 			 char **out)
 {
@@ -441,6 +475,7 @@ unsigned LLVMLinkModules(LLVMModuleRef dest, LLVMModuleRef src, unsigned mode,
 
     return 1;
 }
+#endif
 
 unsigned char *LLVMGetBitcodeFromModule(LLVMModuleRef module, unsigned *lenp)
 {
@@ -505,6 +540,66 @@ int LLVMInlineFunction(LLVMValueRef call)
     return llvm::InlineFunction(cs, unused);
 }
 
+#if HS_LLVM_VERSION >= 300
+// Emits an object file based on the host system's machine specification.
+// The object is emitted to the filename given as an argument.
+bool LLVMAddEmitObjectPass (LLVMModuleRef modRef, const char* filename)
+{
+  llvm::InitializeAllTargetInfos ();
+  llvm::InitializeAllTargets ();
+  llvm::InitializeAllTargetMCs ();
+  llvm::InitializeNativeTarget ();
+  llvm::InitializeAllAsmPrinters ();
+
+  // will be true post 3.0 I think
+  std::string triple = llvm::sys::getDefaultTargetTriple ();
+
+  // std::string triple = llvm::sys::getHostTriple ();
+  std::string err;
+  const llvm::Target* Target = llvm::TargetRegistry::lookupTarget (triple, err);
+
+  std::string cpu = llvm::sys::getHostCPUName ();
+  std::string features = "";
+
+  // llvm::StringMap <bool> featureMap (10);
+  
+  // // this returns false at the moment, but it appears to not make a huge difference
+  // // as the next iteration just doesn't do anything.
+  // llvm::sys::getHostCPUFeatures (featureMap);
+
+  // for (  llvm::StringMap <bool>::const_iterator it = featureMap.begin ();
+  //        it != featureMap.end ();
+  //        ++it) {
+  //   if (it->second) {
+  //     features += it->first.str() + " ";
+  //   }
+  // }
+
+
+  llvm::TargetMachine *machine = 
+    Target->createTargetMachine (triple, cpu, features, llvm::TargetOptions());
+
+
+  llvm::PassManager pass_manager;
+
+  pass_manager.add(new llvm::DataLayout (*machine->getDataLayout()));
+
+  std::string outfile_err;
+  llvm::raw_fd_ostream raw_out (filename, outfile_err);
+  llvm::formatted_raw_ostream out (raw_out);
+
+  if (machine->addPassesToEmitFile (pass_manager, out,
+                                    llvm::TargetMachine::CGFT_ObjectFile,
+                                    false))
+    return false;
+
+  
+  llvm::Module *mod = llvm::unwrap (modRef);
+  pass_manager.run (*mod);
+
+  return true;
+}
+#endif
 
 /* Passes. A few passes (listed below) are used directly from LLVM-C,
  * rest are defined here.
@@ -526,7 +621,9 @@ define_pass( AlwaysInliner )
 define_pass( BlockPlacement )
 define_pass( BreakCriticalEdges )
 define_pass( CodeGenPrepare )
+#if HS_LLVM_VERSION < 303
 define_pass( DbgInfoPrinter )
+#endif
 define_pass( DeadCodeElimination )
 define_pass( DeadInstElimination )
 define_pass( DemoteRegisterToMemory )
@@ -534,13 +631,13 @@ define_pass( DomOnlyPrinter )
 define_pass( DomOnlyViewer )
 define_pass( DomPrinter )
 define_pass( DomViewer )
+define_pass( DependenceAnalysis )
 define_pass( EdgeProfiler )
 define_pass( GlobalsModRef )
 define_pass( InstCount )
 define_pass( InstructionNamer )
 define_pass( LazyValueInfo )
 define_pass( LCSSA )
-define_pass( LoopDependenceAnalysis )
 define_pass( LoopExtractor )
 define_pass( LoopSimplify )
 define_pass( LoopStrengthReduce )
@@ -562,8 +659,56 @@ define_pass( ScalarEvolutionAliasAnalysis )
 define_pass( SingleLoopExtractor )
 define_pass( StripNonDebugSymbols )
 define_pass( UnifyFunctionExitNodes )
+define_pass( Internalize )
 
 /* we support only internalize(true) */
-llvm::ModulePass *createInternalize2Pass() { return llvm::createInternalizePass(true); }
+llvm::ModulePass *createInternalize2Pass() {
+  return llvm::createInternalizePass(); 
+}
+
 define_pass( Internalize2 )
 
+#if HS_LLVM_VERSION < 302
+LLVMBool LLVMPrintModuleToFile(LLVMModuleRef M, const char *Filename,
+                               char **ErrorMessage) {
+  std::string error;
+  llvm::raw_fd_ostream dest(Filename, error);
+  if (!error.empty()) {
+    *ErrorMessage = strdup(error.c_str());
+    return true;
+  }
+
+  llvm::unwrap(M)->print(dest, NULL);
+
+  if (!error.empty()) {
+    *ErrorMessage = strdup(error.c_str());
+    return true;
+  }
+  dest.flush();
+  return false;
+}
+#endif
+
+#if HS_LLVM_VERSION < 303
+LLVMMemoryBufferRef LLVMCreateMemoryBufferWithMemoryRange(
+  const char *InputData,
+  size_t InputDataLength,
+  const char *BufferName,
+  LLVMBool RequiresNullTerminator) {
+
+  return wrap(llvm::MemoryBuffer::getMemBuffer(
+                llvm::StringRef(InputData, InputDataLength),
+                llvm::StringRef(BufferName),
+                RequiresNullTerminator));
+}
+
+LLVMMemoryBufferRef LLVMCreateMemoryBufferWithMemoryRangeCopy(
+  const char *InputData,
+  size_t InputDataLength,
+  const char *BufferName) {
+
+  return wrap(llvm::MemoryBuffer::getMemBufferCopy(
+                llvm::StringRef(InputData, InputDataLength),
+                llvm::StringRef(BufferName)));
+}
+#endif
